/*
 * Copyright@ Samsung Electronics Co. LTD
 *
 * This software is proprietary of Samsung Electronics.
 * No part of this software, either material or conceptual may be copied or distributed, transmitted,
 * transcribed, stored in a retrieval system or translated into any human or computer language in any form by any means,
 * electronic, mechanical, manual or otherwise, or disclosed
 * to third parties without the express written permission of Samsung Electronics.


 * Alternatively, this program is free software in case of open source projec;
 * you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.

*/

#ifndef __BL2_SMC__H__
#define __BL2_SMC__H__

#define SMC_CMD_REG			(-101)

#define SMC_CMD_LOAD_UBOOT		(-230)
#define SMC_CMD_COLDBOOT		(-231)
#define SMC_CMD_WARMBOOT		(-232)
#define SMC_CMD_CHECK_SECOND_BOOT	(-233)
#define SMC_CMD_EMMC_ENDBOOTOP          (-234)
#define SMC_CMD_SDMMC_ENUMERATE         (-235)
#define SMC_CMD_SET_SECURE_REG		(-236)
#define SMC_CMD_SET_SIGNATURE_SIZE	(-238)
#define SMC_CMD_LOAD_ECT_HEAD		(-244)
#define SMC_CMD_LOAD_ECT_ALL		(-245)
#define SMC_CMD_SKIP_ECT		(-246)

/* MACRO for SMC_CMD_REG */
#define SMC_REG_CLASS_SFR_W		(0x1 << 30)
#define SMC_REG_CLASS_SFR_R		(0x3 << 30)
#define SMC_REG_CLASS_MASK		(0x3 << 30)
#define SMC_REG_ID_SFR_W(ADDR)		(SMC_REG_CLASS_SFR_W | ((ADDR) >> 2))
#define SMC_REG_ID_SFR_R(ADDR)		(SMC_REG_CLASS_SFR_R | ((ADDR) >> 2))

#define SMC_SECURE_CONTEXT_BASE		(CONFIG_PHY_IRAM_BASE + 0x4c00)
#ifdef CONFIG_SECURE_BOOT
#define SMC_UBOOT_SIGNATURE_SIZE	256
#else
#define SMC_UBOOT_SIGNATURE_SIZE	0
#endif

typedef struct sdmmc_dev {
	/* for SDMMC */
	unsigned int	image_pos;
	unsigned int	blkcnt;
	unsigned int	base_addr;
} sdmmc_t;

typedef struct emmc_dev {
	/* for eMMC */
	unsigned int	blkcnt;
	unsigned int	base_addr;
} emmc_t;

typedef struct sata_dev {
	/* for SATA */
	unsigned long long	read_sector_of_hdd;
	unsigned int	trans_byte;
	unsigned int	*read_buffer;
	unsigned int	position_of_mem;
} sata_t;

typedef struct sfmc_dev {
	/* for SFMC */
	unsigned int	cs;
	unsigned int	byte_offset;
	unsigned int	byte_size;
	void	*dest_addr;
} sfmc_t;

typedef struct spi_sf_dev {
	/* for SPI SERIAL FLASH */
	unsigned int	flash_read_addr;
	unsigned int	read_size;
	unsigned char	*read_buff;
} spi_sf_t;

typedef struct usb_dev {
	/* for USB */
	unsigned int	base_addr;
	unsigned int	*read_buffer;
} usb_t;

typedef struct ufs_dev {
	/* for UFS */
	unsigned int	start_blk;
	unsigned int	blkcnt;
	unsigned int	dst_addr;
} ufs_t;

/* boot device */
typedef union boot_device_u {
	sdmmc_t 	sdmmc;
	emmc_t		emmc;
	sata_t		sata;
	sfmc_t		sfmc;
	spi_sf_t	spi_sf;
	usb_t		usb;
	ufs_t		ufs;
} boot_device_t;

typedef struct ld_image_info {
	/* for Signature */
	unsigned int	image_base_addr;
	unsigned int	size;
	unsigned int	secure_context_base;
	unsigned int	signature_size;
	boot_device_t bootdev;
} image_info;

typedef struct reg_val {
	unsigned int	addr;
	unsigned int	val;
} reg_val_t;

typedef struct reg_arr {
	reg_val_t	set0;
	reg_val_t	set1;
	reg_val_t	set2;
	reg_val_t	set3;
	reg_val_t	set4;
	reg_val_t	set5;
	reg_val_t	set6;
	reg_val_t	set7;
	reg_val_t	set8;
	reg_val_t	set9;
} reg_arr_t;

#endif	/* __BL2_SMC__H__ */
