/*
 * Copyright@ Samsung Electronics Co. LTD
 *
 * This software is proprietary of Samsung Electronics.
 * No part of this software, either material or conceptual may be copied or distributed, transmitted,
 * transcribed, stored in a retrieval system or translated into any human or computer language in any form by any means,
 * electronic, mechanical, manual or otherwise, or disclosed
 * to third parties without the express written permission of Samsung Electronics.


 * Alternatively, this program is free software in case of open source project
 * you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.

*/

#ifndef __PMIC_H__
#define __PMIC_H__

/* S2MPU06 slave address */
#define S2MPU06_PM_ADDR	0x1

/* S2MPU06 Register Address */
#define S2MPU06_PM_RTC_BUF	0x0B
#define S2MPU06_PM_CTRL1	0x0C
#define S2MPU06_PM_CTRL3	0x0E
#define S2MPU06_PM_BUCK1_OUT1	0x14
#define S2MPU06_PM_BUCK1_OUT2	0x15
#define S2MPU06_PM_BUCK1_OUT3	0x16
#define S2MPU06_PM_LDO1_CTRL1	0x1E
#define S2MPU06_PM_LDO1_CTRL2	0x1F
#define S2MPU06_PM_LDO6_CTRL1	0x24
#define S2MPU06_PM_LDO6_CTRL2	0x25
#define S2MPU06_PM_LDO9_CTRL	0x28
#define S2MPU06_PM_LDO10_CTRL	0x29
#define S2MPU06_PM_LDO11_CTRL	0x2A
#define S2MPU06_PM_LDO12_CTRL	0x2B

/*
 * LDO OUT mask [5:0]
 */
#define LDOOUT_MASK		0x3F

/*
 * TCXO
 */
#define TCXO_EN			0x40
#define LDOEN_MASK		0xC0

/*
 * RTC_BUF
 */
#define PERI_32KHZ_EN		(0x1 << 1)
#define AP_32KHZ_EN		(0x1 << 0)

/*
 * CTRL1
 */
#define MRSTBEN		(0x1 << 4)

/*
 * CTRL3
 */
#define WRSTEN			(0x1 << 4)

#define VDD_BASE_VOLT		50000
#define VDD_VOLT_STEP		625
#define MIN_VOLT		500
#define MAX_RD_VAL		0xB0
#define RD_BUCK_VOLT(x)	(((x > MAX_RD_VAL) ? 0 : \
				(((x - 0x10) * VDD_VOLT_STEP) + VDD_BASE_VOLT) / 100))
#define WR_BUCK_VOLT(x)	((x < MIN_VOLT) ? 0x10 : \
				((((x) * 100) - VDD_BASE_VOLT) / VDD_VOLT_STEP + 0x10))

extern void pmic_enable_peric_dev(void);

#endif /*__PMIC_H__*/

