/*
 * Copyright@ Samsung Electronics Co. LTD
 *
 * This software is proprietary of Samsung Electronics.
 * No part of this software, either material or conceptual may be copied or distributed, transmitted,
 * transcribed, stored in a retrieval system or translated into any human or computer language in any form by any means,
 * electronic, mechanical, manual or otherwise, or disclosed
 * to third parties without the express written permission of Samsung Electronics.


 * Alternatively, this program is free software in case of open source project
 * you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.

*/

#include <config.h>
#include <version.h>
#include <common.h>
#include <asm/io.h>
#include <asm/arch/cpu.h>
#include <asm/arch/smc.h>
#include "asv.h"

#ifndef CONFIG_SG_BINARY

#define CHIPID_BASE	(void *)(0x10100000)
#define OTP_CON_TOP_BASE	(void *)(0x100D0000)

struct chip_info chip_info;

static void CHIPID_DecTo36(unsigned int in, char *p)
{
	unsigned int nMod;
	unsigned int i;

	for (i = 4; i >= 1; i--) {
		nMod = in % 36;
		in /= 36;
		p[i] = (nMod < 10) ? (nMod + '0') : (nMod - 10 + 'A');
	}

	p[0] = 'N';
	p[5] = 0;
}

static unsigned int CHIPID_ReverseValue(unsigned int uValue, unsigned int nBitCnt)
{
	unsigned int uTemp, uRetValue = 0;
	unsigned int i;

	for (i = 0; i < nBitCnt; i++) {
		uTemp = (uValue >> i) & 0x1;
		uRetValue += uTemp << ((nBitCnt - 1) - i);
	}

	return uRetValue;
}

void init_chip_info(void)
{
	unsigned int uReverseID0, uReverseID1;
	unsigned int uBits, uTemp;

	/* sample Info */
	uBits = __raw_readl(CHIPID_BASE + 0x0014);
	uReverseID0 = CHIPID_ReverseValue(uBits, 32);
	uBits = __raw_readl(CHIPID_BASE + 0x0018);
	uReverseID1 = CHIPID_ReverseValue(uBits, 16);

	uTemp = (uReverseID0 >> 11) & 0x1FFFFF;
	CHIPID_DecTo36(uTemp, chip_info.szWaferID);

	chip_info.nWaferNum = (uReverseID0 >> 6) & 0x1F;

	uTemp = uReverseID0 & 0x3F;
	chip_info.nWaferX = ((uTemp << 2) + (uReverseID1 >> 14)) & 0xFF;
	chip_info.nWaferY = (uReverseID1 >> 6) & 0xFF;

	/* ASV Group */
	uBits = (unsigned int)exynos_smc_read(SMC_CMD_REG, SMC_REG_ID_SFR_R((unsigned long long)(OTP_CON_TOP_BASE + 0x9000)));
	chip_info.nAsv_CPUCL0 = ((unsigned char)((char)((uBits >> 0) & 0xF) + (char)((uBits >> 4) & 0xF))) & 0xF;

	uBits = (unsigned int)exynos_smc_read(SMC_CMD_REG, SMC_REG_ID_SFR_R((unsigned long long)(OTP_CON_TOP_BASE + 0x9004)));
	chip_info.nAsv_G3D = ((unsigned char)((char)((uBits >> 0) & 0xF) + (char)((uBits >> 4) & 0xF))) & 0xF;
	chip_info.nAsv_MIF = ((unsigned char)((char)((uBits >> 16) & 0xF) + (char)((uBits >> 20) & 0xF))) & 0xF;

	uBits = (unsigned int)exynos_smc_read(SMC_CMD_REG, SMC_REG_ID_SFR_R((unsigned long long)(OTP_CON_TOP_BASE + 0x9008)));
	chip_info.nAsv_INT = ((unsigned char)((char)((uBits >> 0) & 0xF) + (char)((uBits >> 4) & 0xF))) & 0xF;
	chip_info.nAsv_CAM_DISP = ((unsigned char)((char)((uBits >> 16) & 0xF) + (char)((uBits >> 20) & 0xF))) & 0xF;

	uBits = (unsigned int)exynos_smc_read(SMC_CMD_REG, SMC_REG_ID_SFR_R((unsigned long long)(OTP_CON_TOP_BASE + 0x900C)));
	chip_info.nAsv_TABLE = (uBits >> 0) & 0x7F;
	chip_info.nAsv_CP = ((unsigned char)((char)((uBits >> 16) & 0xF) + (char)((uBits >> 20) & 0xF))) & 0xF;
}
#endif
