/*
 * FUEL GAUGE GPIO External I2C Emulation
 */
#include <asm/types.h>
#include <asm/arch/pmic_s2mpw01.h>
#include "../../../drivers/speedy/speedy.h"

#define max(a, b)   ((a) > (b) ? (a) : (b))
#define min(a, b)   ((a) < (b) ? (a) : (b))

#define FUELGAUGE_POWER_CHECK_COUNT     20
#define FG_BOOTABLE_VOL                 3400
#define FG_POWER_CHECK_COUNT            50
#define FG_POWER_CHECK_DELAY            1000
#define AVER_SAMPLE_CNT                 5

#define VALID_CHARGER   0x01
#define INVALID_CHARGER 0x00

static unsigned char model_param1_300ma[] = {
	0x2C, 0x01, 0xFF, 0xFF
};

static unsigned char model_param2_300ma[] = {
	0x7A, 0x0A, 0x0A, 0x0A, 0x9B, 0x09, 0x36, 0x09, 0xD5, 0x08,
	0x88, 0x08, 0x04, 0x08, 0xC1, 0x07, 0x76, 0x07, 0x1F, 0x07,
	0xDD, 0x06, 0xAC, 0x06, 0x85, 0x06, 0x64, 0x06, 0x49, 0x06,
	0x2F, 0x06, 0x0E, 0x06, 0xDC, 0x05, 0x96, 0x05, 0x88, 0x05,
	0x03, 0x05, 0xA9, 0x02, 0x69, 0x08, 0x00, 0x08, 0x97, 0x07,
	0x2E, 0x07, 0xC6, 0x06, 0x5D, 0x06, 0xF4, 0x05, 0x8B, 0x05,
	0x23, 0x05, 0xBA, 0x04, 0x51, 0x04, 0xE8, 0x03, 0x7F, 0x03,
	0x17, 0x03, 0xAE, 0x02, 0x45, 0x02, 0xDD, 0x01, 0x74, 0x01,
	0x0B, 0x01, 0xA2, 0x00, 0x3A, 0x00, 0xE9, 0x0F
};

static unsigned char model_param3_300ma[] = {
	0x70, 0x70, 0x6A, 0x64, 0x63, 0x57, 0x4F, 0x53, 0x47, 0x42,
	0x42, 0x41, 0x44, 0x4B, 0x51, 0x5E, 0x6F, 0x83, 0xA1, 0xCD,
	0xFF, 0x9A
};

static unsigned char model_param1_380ma[] = {
	0x60, 0x7C, 0x1, 0x76, 0xAE
};

static unsigned char model_param2_380ma[] = {
	0xE1, 0xA, 0x79, 0xA, 0xE4, 0x9, 0x78, 0x9, 0x11, 0x9,
	0xAC, 0x8, 0x56, 0x8, 0xE0, 0x7, 0xA2, 0x7, 0x47, 0x7,
	0xF5, 0x6, 0xBD, 0x6, 0x92, 0x6, 0x6E, 0x6, 0x51, 0x6,
	0x35, 0x6, 0x14, 0x6, 0xE9, 0x5, 0xA3, 0x5, 0x8A, 0x5,
	0x7B, 0x4, 0xA4, 0x2, 0x0, 0x8, 0x9D, 0x7, 0x3B, 0x7,
	0xD8, 0x6, 0x76, 0x6, 0x13, 0x6, 0xB1, 0x5, 0x4E, 0x5,
	0xEC, 0x4, 0x89, 0x4, 0x27, 0x4, 0xC4, 0x3, 0x62, 0x3,
	0xFF, 0x2, 0x9D, 0x2, 0x3A, 0x2, 0xD8, 0x1, 0x75, 0x1,
	0x13, 0x1, 0xB0, 0x0, 0x4E, 0x0, 0xEB, 0xF
};

static unsigned char model_param3_380ma[] = {
	0xA2, 0x9C, 0x97, 0x94, 0x93, 0x90, 0x8F, 0x92, 0x94, 0x96,
	0x66, 0x66, 0x66, 0x66, 0x9A, 0x9A, 0x9A, 0xCD, 0xCD, 0xE6,
	0xE6, 0xE6
};

static unsigned char model_param1_550ma[] = {
	0x26, 0x02, 0xFF, 0xFF
};

static unsigned char model_param2_550ma[] = {
	0x7A, 0x0A, 0x0A, 0x0A, 0x9B, 0x09, 0x36, 0x09, 0xD5, 0x08,
	0x88, 0x08, 0x04, 0x08, 0xC1, 0x07, 0x76, 0x07, 0x1F, 0x07,
	0xDD, 0x06, 0xAC, 0x06, 0x85, 0x06, 0x64, 0x06, 0x49, 0x06,
	0x2F, 0x06, 0x0E, 0x06, 0xDC, 0x05, 0x96, 0x05, 0x88, 0x05,
	0x03, 0x05, 0xA9, 0x02, 0x69, 0x08, 0x00, 0x08, 0x97, 0x07,
	0x2E, 0x07, 0xC6, 0x06, 0x5D, 0x06, 0xF4, 0x05, 0x8B, 0x05,
	0x23, 0x05, 0xBA, 0x04, 0x51, 0x04, 0xE8, 0x03, 0x7F, 0x03,
	0x17, 0x03, 0xAE, 0x02, 0x45, 0x02, 0xDD, 0x01, 0x74, 0x01,
	0x0B, 0x01, 0xA2, 0x00, 0x3A, 0x00, 0xE9, 0x0F
};

static unsigned char model_param3_550ma[] = {
	0x70, 0x70, 0x6A, 0x64, 0x63, 0x57, 0x4F, 0x53, 0x47, 0x42,
	0x42, 0x41, 0x44, 0x4B, 0x51, 0x5E, 0x6F, 0x83, 0xA1, 0xCD,
	0xFF, 0x9A
};

static u16 ReadFGRegisterWord(u8 addr)
{
	u8 msb = 0;
	u8 lsb = 0;

	PMIC_FUELGAUGE_READ(addr++, &lsb);
	PMIC_FUELGAUGE_READ(addr, &msb);
#if defined(CONFIG_BUILD_TYPE_ENG)
	printf("%s: REG(0x%x) = 0x%x%x\n", __func__, addr, msb, lsb);
#endif
	return ((msb << 8) | lsb);
}

int fg_reset_soc(void)
{
	u8 temp;

	/* fuelgauge reset soc */
	PMIC_FUELGAUGE_READ(0X17, &temp);
	temp |= 0x01;
	PMIC_FUELGAUGE_WRITE(0X17, temp);

	printf("fuelgauge : quick start is executed!\n");
	return 1;
}

u32 fuel_gauge_read_vcell(void)
{
	u16 data;
	u32 vcell = 0;
	/* get vcell value, return vcell value in mV unit */
	data = ReadFGRegisterWord(S2MPW01_FG_REG_RVBAT);

	vcell = (data * 1000) >> 13;
	printf("fuelgauge : VCELL %dmV\n", vcell);
	return vcell;
}

u32 get_avg_voltage(void)
{
	u32 vcell_data = 0;
	u32 vcell_max = 0;
	u32 vcell_min = 0;
	u32 vcell_total = 0;
	u32 i;

	for (i = 0; i < AVER_SAMPLE_CNT; i++) {
		vcell_data = fuel_gauge_read_vcell();

		if (i != 0) {
			if (vcell_data > vcell_max)
				vcell_max = vcell_data;
			else if (vcell_data < vcell_min)
				vcell_min = vcell_data;
		} else {
			vcell_max = vcell_data;
			vcell_min = vcell_data;
		}
		vcell_total += vcell_data;
	}

	return (vcell_total - vcell_max - vcell_min) / (AVER_SAMPLE_CNT-2);
}

u32 fuel_gauge_read_soc(void)
{

	u16 compliment;
	u32 rsoc;

	/* get soc value */
	compliment = ReadFGRegisterWord(S2MPW01_FG_REG_RSOC);

	/* data[] store 2's compliment format number */
	if (compliment & (0x1 << 15)) {
		/* Negative */
		rsoc = ((~compliment) & 0xFFFF) + 1;
		rsoc = (rsoc * (-10000)) / (0x1 << 12);
	} else {
		rsoc = compliment & 0x7FFF;
		rsoc = ((rsoc * 10000) / (0x1 << 12));
	}

	printf("fuelgauge : capacity (0x%x:%d)\n", compliment, rsoc);

	return min(rsoc, 10000) / 10;

}

void S2MPW01_Register_dump(void)
{
	/* for debugging */
	u8 reg_data;
	u8 addr;

	/* check register setting for debugging */

	printf("%s: ", __func__);

	for (addr = 0x00; addr <= 0x17; addr++) {
		PMIC_FUELGAUGE_READ(addr, &reg_data);
		printf("0x%02x:0x%02x, ", addr, reg_data);
	}
	printf("\n");
}

u16 fuel_gauge_get_ocv(void)
{
	u16 data;
	u32 ocv;
	/* get ocv value */
	data = ReadFGRegisterWord(S2MPW01_FG_REG_ROCV);
	ocv = (data * 1000) >> 13;
	dbg("fuelgauge : OCV %dmV\n", ocv);
	return ocv;
}

#ifdef FG_DEBUG
#define COLOR_RED		0x00ff0000
#define COLOR_GREEN	0x0000ff00
#define COLOR_BLUE		0x000000ff
#define COLOR_BLACK		0x00000000
#define COLOR_WHITE		0x00ffffff
extern void lcd_print(s32 color, const s8 *fmt, ...);

void fg_debug_print(void)
{
	lcd_print(COLOR_GREEN, "S = %d%%, TS = %d%%, V = %dmv",
		g_soc, g_table_soc, g_vcell);
	lcd_print(COLOR_GREEN, "R = %d, Q = %d, D = %d%%",
		g_rbc, g_quick_start, g_diff);
	lcd_print(COLOR_GREEN, "New S = %d%%, V = %dmv",
		g_soc2, g_vcell2);
}
#endif

/* For featuring e7270pro board battery parameter setting */
#define BOARD_E7270PRO
#ifndef BOARD_E7270PRO
void set_fuel_gauge_battery_parameter(void)
{
	u8 addr;
	u8 temp;
	int i;
	int len;
	int system_rev = 0x5; /* For temporary set */

	if (system_rev <= 0x04) {
		addr = 0x0c;
		len = 4;
	} else {
		addr = 0x0b;
		len = 5;
	}

	for (i = 0; i < len; i++) {
		if (system_rev <= 0x04)
			PMIC_FUELGAUGE_WRITE(i+addr, model_param1_300ma[i]);
		else
			PMIC_FUELGAUGE_WRITE(i+addr, model_param1_380ma[i]);
	}

	/* 0x2C[2:0]=100 sampling period : 16s */
	PMIC_FUELGAUGE_READ(0x2C, &temp);
	temp &= ~0x07;
	temp |= 0x04;
	PMIC_FUELGAUGE_WRITE(0x2C, temp);

	addr = 0x2E;
	for (i = 0; i < 88; i++) {
		if (system_rev <= 0x04)
			PMIC_FUELGAUGE_WRITE(i+addr, model_param2_300ma[i]);
		else
			PMIC_FUELGAUGE_WRITE(i+addr, model_param2_380ma[i]);

	}

	addr = 0xBC;
	for (i = 0; i < 22; i++) {
		if (system_rev <= 0x04)
			PMIC_FUELGAUGE_WRITE(i+addr, model_param3_300ma[i]);
		else
			PMIC_FUELGAUGE_WRITE(i+addr, model_param3_380ma[i]);
	}

	PMIC_FUELGAUGE_READ(0x19, &temp);
	temp &= 0x0F;
	temp |= 0xF0;
	PMIC_FUELGAUGE_WRITE(0x19, temp);
	PMIC_FUELGAUGE_READ(0x28, &temp);
	temp &= 0x0F;
	PMIC_FUELGAUGE_WRITE(0x28, temp);
	PMIC_FUELGAUGE_READ(0x2B, &temp);
	temp &= 0x0F;
	PMIC_FUELGAUGE_WRITE(0x2B, temp);

	printf("%s: apply ocv table\n", __func__);
}
#else
void set_fuel_gauge_battery_parameter(void)
{
	u8 addr;
	u8 temp;
	int i;
	int len;

	addr = 0x0c;
	len = 4;

	for (i = 0; i < len; i++)
		PMIC_FUELGAUGE_WRITE(i+addr, model_param1_550ma[i]);

	/* 0x2C[2:0]=100 sampling period : 16s */
	PMIC_FUELGAUGE_READ(0x2C, &temp);
	temp &= ~0x07;
	temp |= 0x04;
	PMIC_FUELGAUGE_WRITE(0x2C, temp);

	addr = 0x2E;
	for (i = 0; i < 88; i++)
		PMIC_FUELGAUGE_WRITE(i+addr, model_param2_550ma[i]);

	addr = 0xBC;
	for (i = 0; i < 22; i++)
		PMIC_FUELGAUGE_WRITE(i+addr, model_param3_550ma[i]);

	PMIC_FUELGAUGE_READ(0x19, &temp);
	temp &= 0x0F;
	temp |= 0xF0;
	PMIC_FUELGAUGE_WRITE(0x19, temp);
	PMIC_FUELGAUGE_READ(0x28, &temp);
	temp &= 0x0F;
	PMIC_FUELGAUGE_WRITE(0x28, temp);
	PMIC_FUELGAUGE_READ(0x2B, &temp);
	temp &= 0x0F;
	PMIC_FUELGAUGE_WRITE(0x2B, temp);

	printf("%s: apply ocv table\n", __func__);
}
#endif

void set_fuel_gauge_model_data(void)
{
	u8 temp;
	/* for offset value*/
	unsigned char reg29;
	unsigned char reg2A;
	unsigned char reg2B;
	unsigned char reg29_t;
	unsigned char reg2A_t;
	unsigned char reg2B_t;
	unsigned int dest;
	unsigned int offset;
	unsigned int ocv;

	set_fuel_gauge_battery_parameter();

	offset = 0xA3D; /* Measured vbat - FG vbat */

	/* backup default offset value */
	PMIC_FUELGAUGE_READ(0x29, &reg29);
	PMIC_FUELGAUGE_READ(0x2A, &reg2A);
	PMIC_FUELGAUGE_READ(0x2B, &reg2B);
	printf("%s: 1. reg29:0x%02x, reg2A:0x%02x, reg2B:0x%02x, offset:0x%x\n",
			__func__, reg29, reg2A, reg2B, offset);

	dest = ((reg2B << 16) & 0x0F0000) | ((reg2A << 8) & 0xFF00) | (reg29 & 0xFF);
	if ((dest > (0x7FFFF - offset)) && (dest < 0x80000))
	   dest = 0x7FFFF;
	 else
	   dest = dest + offset;

	/* change offset value */
	PMIC_FUELGAUGE_WRITE(0x2B, ((dest >> 16) & 0x0F) | (reg2B & 0xF0));
	PMIC_FUELGAUGE_WRITE(0x2A, (dest >> 8) & 0xFF);
	PMIC_FUELGAUGE_WRITE(0x29, dest & 0xFF);

	PMIC_FUELGAUGE_READ(0x29, &reg29_t);
	PMIC_FUELGAUGE_READ(0x2A, &reg2A_t);
	PMIC_FUELGAUGE_READ(0x2B, &reg2B_t);
	printf("%s: 2. reg29:0x%02x, reg2A:0x%02x, reg2B:0x%02x\n", __func__, reg29_t, reg2A_t, reg2B_t);

	/* initial soc dump_done */
	PMIC_FUELGAUGE_READ(S2MPW01_FG_REG_START, &temp);
	temp |= 0x33;
	PMIC_FUELGAUGE_WRITE(S2MPW01_FG_REG_START, temp);
	mdelay(500);

	/* recover default offset value */
	PMIC_FUELGAUGE_WRITE(0x29, reg29);
	PMIC_FUELGAUGE_WRITE(0x2A, reg2A);
	PMIC_FUELGAUGE_WRITE(0x2B, reg2B);

	PMIC_FUELGAUGE_READ(0x29, &reg29);
	PMIC_FUELGAUGE_READ(0x2A, &reg2A);
	PMIC_FUELGAUGE_READ(0x2B, &reg2B);
	printf("%s: 3. reg29:0x%02x, reg2A:0x%02x, reg2B:0x%02x\n", __func__, reg29, reg2A, reg2B);
}

void reset_fuel_gauge(void)
{
#if 0
	u8 ocv_apply, temp;
	/*To apply ocv table(Not dump_done) after download, set 0x1D[7]=1 */
	PMIC_FUELGAUGE_READ(0x1D, &ocv_apply);
	temp = (ocv_apply | 0x80);
	PMIC_FUELGAUGE_WRITE(0x1D, temp);

	printf("%s: ocv_apply 0x1D: 0x%02x, 0x%02x\n", __func__, ocv_apply, temp);
#endif
	printf("%s: Do nothing\n", __func__);

}

void init_pre_fuel_gauge(void)
{
	/* if need pre initialization for fuelgauge */
	u8 por_state, temp, temp1;
	u32 vcell;

	PMIC_FUELGAUGE_READ(0x17, &por_state); /* reset condition : 0x1F[4] */
	printf("%s: POR state = 0x%02x\n", __func__, por_state);

	if (por_state & 0x10) {
		/* initial soc dump_done */
		PMIC_FUELGAUGE_READ(S2MPW01_FG_REG_START, &temp);
		temp |= 0x03;
		PMIC_FUELGAUGE_WRITE(S2MPW01_FG_REG_START, temp);
		/* It must be needed for Voltage recovery time */
		mdelay(100);
	} else {
		vcell = fuel_gauge_read_vcell();
		if (vcell < 3400) {
			/* initial soc dump_done */
			PMIC_FUELGAUGE_READ(S2MPW01_FG_REG_START, &temp);
			temp |= 0x33;
			PMIC_FUELGAUGE_WRITE(S2MPW01_FG_REG_START, temp);

			/* It must be needed for Voltage recovery time */
			mdelay(100);

			PMIC_FUELGAUGE_READ(0x2C, &temp);
			temp |= 0x08;
			PMIC_FUELGAUGE_WRITE(0x2C, temp);
			printf("%s: dump restart !!!\n", __func__);
		}
	}
}

#if 0
/* init fuelgauge driver */
void init_fuel_gauge(void)
{
	u8 por_state, ocv_apply, temp;
	u32 soc, vcell, ocv;
	u32 chg_state = 0;

	PMIC_FUELGAUGE_READ(0x2C, &temp);
	printf("%s: FG 0x2C = 0x%02x\n", __func__, temp);
	PMIC_FUELGAUGE_READ(0x17, &por_state); /* reset condition : 0x1F[4] */
	printf("%s: POR state = 0x%02x\n", __func__, por_state);

	soc = fuel_gauge_read_soc();

	if ((por_state & 0x10) || (soc < 10) || (temp & 0x08)) {
		set_fuel_gauge_model_data();

		por_state &= ~0x10;
		PMIC_FUELGAUGE_WRITE(0x17, por_state);
		dbg("%s: Fuelgauge Reset\n", __func__);

		/* FG 0x2C[3]=0 */
		PMIC_FUELGAUGE_READ(0x2C, &temp);
		temp &= ~0x08;
		PMIC_FUELGAUGE_WRITE(0x2C, temp);
	}
	soc = fuel_gauge_read_soc();
	vcell = get_avg_voltage();
	ocv = fuel_gauge_get_ocv();
	printf("%s: vcell(%d) ocv(%d)\n", __func__, vcell, ocv);
	printf("%s: done\n", __func__);
}
#endif

void init_fuel_gauge(void)
{
	u8 por_state, ocv_apply, temp;
	u32 soc, vcell, ocv;
	u32 chg_state = 0;
	/* init fuelgauge driver */

	PMIC_FUELGAUGE_READ(0x2C, &temp);
	printf("%s: FG 0x2C = 0x%02x\n", __func__, temp);
	PMIC_FUELGAUGE_READ(0x17, &por_state); /* reset condition : 0x1F[4] */
	printf("%s: POR state = 0x%02x\n", __func__, por_state);

	soc = fuel_gauge_read_soc();

	if ((por_state & 0x10) || (soc < 10) || (temp & 0x08)) {
		set_charger_state(1, 0);

		set_fuel_gauge_model_data();

		set_charger_state(1, 1);

		por_state &= ~0x10;
		PMIC_FUELGAUGE_WRITE(0x17, por_state);
		dbg("%s: Fuelgauge Reset\n", __func__);

		/* FG 0x2C[3]=0 */
		PMIC_FUELGAUGE_READ(0x2C, &temp);
		temp &= ~0x08;
		PMIC_FUELGAUGE_WRITE(0x2C, temp);
	}

	soc = fuel_gauge_read_soc();
	vcell = get_avg_voltage();
	ocv = fuel_gauge_get_ocv();
	printf("%s: vcell(%d) ocv(%d)\n", __func__, vcell, ocv);
	printf("%s: done\n", __func__);
}

