
#include <common.h>

#include <openssl/nid.h>
#include <openssl/rsa.h>
#include <openssl/bn.h>

#include <asm/arch/smc.h>
#include "cm_api.h"

int get_hash(uint32_t addr, uint32_t size, uint32_t alg, uint32_t *hash)
{
	int i;
	unsigned int ret;
	HASH_INFO info_hash;

	info_hash.hash_addr = addr;
	info_hash.result_hash = (unsigned int)hash;
	info_hash.total_size = size;
	info_hash.kindofhash = alg;
	ret = exynos_smc(SMC_CMD_HASH, (u64)&info_hash, 0, SHA_DIGEST);
#if 0
	printf("hash calc result %d\n", ret);
	if (ret == 0) {
		p = (unsigned char *)hash;
		for (i = 0; i < 32; i++) {
			printf("%02x ", *p);
			p++;
		}
		printf("\n");
	}
#endif
	return ret;
}

uint8_t *decode_length(uint8_t *p, uint32_t *out_len)
{
	int i;
	uint32_t length = 0;
	uint8_t num_octets;

	num_octets = *p++;
	if ((num_octets & 0x80) == 0x00) {
		*out_len = num_octets;
		return p;
	}
	num_octets &= 0x7f;
	if (num_octets > 2) {
		printf("num_octets %d\n", num_octets);
		return NULL;
	}
	for (i = 0; i < num_octets; ++i) {
		length <<= 8;
		length += *p;
		p++;
	}
	*out_len = length;
	return p;
}

#define ASN1_TAG_SEQUENCE	0x30
#define ASN1_TAG_INTEGER	0x02
#define ASN1_TAG_BITSTRING	0x03
#define ASN1_TAG_OCTETSTR	0x04
#define ASN1_TAG_PRINTABLE	0x13

uint8_t *asn1_constructed_skip_all(uint8_t *p)
{
	uint32_t length;

	while((*p & 0xe0) == 0xa0) {
		p++;
		if ((p = decode_length(p, &length)) == NULL)
			return NULL;
		p += length;
	}
	return p;
}

uint8_t *asn1_obj_get(uint8_t tag, uint8_t *p, uint32_t *len)
{
	if (*p != tag) {
		printf("tag not match %02x %02x\n", *p, tag);
		return NULL;
	}
	p++;
	p = decode_length(p, len);
	return p;
}

uint8_t *asn1_obj_skip(uint8_t tag, uint8_t *p)
{
	uint32_t length;

	if ((p = asn1_obj_get(tag, p, &length)) == NULL)
		return NULL;
	p += length;
	return p;
}

uint8_t *asn1_obj_get_skip(uint8_t tag, uint8_t *p, uint8_t **pval, uint32_t *len)
{
	if (*p != tag) {
		printf("tag not match %02x %02x\n", *p, tag);
		return NULL;
	}
	p++;
	if ((p = decode_length(p, len)) == NULL)
		return NULL;
	*pval = p;
	p += (*len);
	return p;
}

// ObjectIdentifier 1.2.840.113549.1.1.11 (sha256withRSAEncryption)
static uint8_t oid_sha256withrsa[] = {
	0x2a, 0x86, 0x48, 0x86, 0xf7, 0x0d, 0x01, 0x01, 0x0b
};

static uint8_t oemkey[] = {
#if 1	// Google's key for medaka
//0x30,0x82,0x01,0x0A,
// modules
0x02,0x82,0x01,0x01,
0x00,0xb3,0x20,0x12,0xd7,0x65,0x77,0xda,0x34,0xc2,0x29,0xcd,0xb6,0x0d,0x39,0x34,
0x32,0xa0,0xc1,0xb2,0x27,0xfd,0x89,0x3c,0xd0,0xca,0x73,0x9e,0x51,0x19,0x21,0x97,
0x09,0x64,0x19,0xc3,0x87,0x4f,0x05,0x74,0xc8,0xa4,0x60,0x6b,0x0c,0xb8,0x52,0x30,
0x24,0x88,0x25,0x96,0x52,0x77,0x16,0x4e,0x89,0x7b,0xf3,0x25,0x82,0x54,0x0b,0xdb,
0xcd,0xad,0x92,0x32,0xe1,0xea,0x14,0x9d,0xe1,0x10,0x38,0x75,0x28,0x4c,0x49,0x43,
0x7d,0x61,0x0c,0x4f,0xb4,0x5f,0xc0,0x5c,0xef,0x8c,0x45,0x15,0x1d,0xfc,0xfa,0x33,
0x84,0x36,0x92,0x5c,0x39,0x4c,0xdf,0x20,0x5b,0xc2,0xf4,0x5a,0x9a,0xa3,0xf6,0x4b,
0xaf,0x48,0xc5,0x72,0x95,0x84,0x40,0x8d,0xce,0x9f,0x50,0x63,0x6b,0x60,0x3b,0x73,
0x41,0x41,0xfb,0xf6,0x8a,0x96,0x78,0x8c,0xd3,0x7c,0xc5,0x2c,0xa9,0x0b,0xeb,0xf2,
0x09,0x36,0x1c,0x87,0x9f,0x7d,0x7e,0x15,0xae,0x3d,0x16,0xb9,0x9c,0xb6,0x6c,0x47,
0x21,0x80,0x8e,0x60,0x55,0xf9,0x8c,0xe3,0x3e,0x58,0x11,0xf0,0x05,0xd1,0x63,0x6d,
0x61,0x3f,0xb4,0x1a,0x54,0x5b,0xf0,0xff,0x8c,0xa8,0x7f,0xd2,0x99,0x79,0xd9,0x6f,
0x2a,0x41,0x25,0x3d,0x33,0x9d,0x50,0xc4,0xf0,0x10,0xe8,0x27,0xdf,0x0f,0xf6,0x7e,
0x93,0xf8,0x2c,0x3e,0x3f,0xee,0x96,0x31,0x48,0x4a,0x0d,0x8b,0x89,0xfb,0x5d,0x4d,
0xce,0x38,0x2c,0xda,0x6b,0xa0,0xba,0xae,0x10,0x29,0x13,0x95,0x11,0x95,0x3d,0xed,
0xf9,0xb6,0x1c,0xa1,0x82,0xa8,0x18,0xf2,0x9c,0x4b,0x3c,0xa5,0x1f,0x4d,0x82,0x74,
0xe3,
// exponent
0x02,0x03,0x01,0x00,0x01
#else	// AOSP key
//0x30,0x82,0x01,0x0A,
// modules
0x02,0x82,0x01,0x01,
0x00,0xE8,0xEB,0x78,0x4D,0x2F,0x4D,0x54,0x91,0x7A,0x7B,0xB3,0x3B,0xDB,0xE7,0x69,
0x67,0xE4,0xD1,0xE4,0x33,0x61,0xA6,0xF4,0x82,0xAA,0x62,0xEB,0x10,0x33,0x8B,0xA7,
0x66,0x0F,0xEB,0xA0,0xA0,0x42,0x89,0x99,0xB3,0xE2,0xB8,0x4E,0x43,0xC1,0xFD,0xB5,
0x8A,0xC6,0x7D,0xBA,0x15,0x14,0xBB,0x47,0x50,0x33,0x8E,0x9D,0x2B,0x8A,0x1C,0x2B,
0x13,0x11,0xAD,0xC9,0xE6,0x1B,0x1C,0x9D,0x16,0x7E,0xA8,0x7E,0xCD,0xCE,0x0C,0x93,
0x17,0x3A,0x4B,0xF6,0x80,0xA5,0xCB,0xFC,0x57,0x5B,0x10,0xF7,0x43,0x6F,0x1C,0xDD,
0xBB,0xCC,0xF7,0xCA,0x4F,0x96,0xEB,0xBB,0x9D,0x33,0xF7,0xD6,0xED,0x66,0xDA,0x43,
0x70,0xCE,0xD2,0x49,0xEE,0xFA,0x2C,0xCA,0x6A,0x4F,0xF7,0x4F,0x8D,0x5C,0xE6,0xEA,
0x17,0x99,0x0F,0x35,0x50,0xDB,0x40,0xCD,0x11,0xB3,0x19,0xC8,0x4D,0x55,0x73,0x26,
0x5A,0xE4,0xC6,0x3A,0x48,0x3A,0x53,0xED,0x08,0xD9,0x37,0x7B,0x2B,0xCC,0xAF,0x50,
0xC5,0xA1,0x01,0x63,0xCF,0xA4,0xA2,0xED,0x54,0x7F,0x6B,0x00,0xBE,0x53,0xCE,0x36,
0x0D,0x47,0xDD,0xA2,0xCD,0xD2,0x9C,0xCF,0x70,0x23,0x46,0xC2,0x37,0x09,0x38,0xED,
0xA6,0x25,0x40,0x04,0x67,0x97,0xD1,0x37,0x23,0x45,0x2B,0x99,0x07,0xB2,0xBD,0x10,
0xAE,0x7A,0x1D,0x5F,0x8E,0x14,0xD4,0xBA,0x23,0x53,0x4F,0x8D,0xD0,0xFB,0x14,0x84,
0xA1,0xC8,0x69,0x6A,0xA9,0x97,0x54,0x3A,0x40,0x14,0x65,0x86,0xA7,0x6E,0x98,0x1E,
0x4F,0x93,0x7B,0x40,0xBE,0xAE,0xBA,0xA7,0x06,0xA6,0x84,0xCE,0x91,0xA9,0x6E,0xEA
,0x49,
// exponent
0x02,0x03,0x01,0x00,0x01
#endif
};

static int is_oemkey;
int is_oem_key(void)
{
	return is_oemkey;
}

static uint8_t *prsa_n, *prsa_e, *psig;
static uint32_t rsa_n_len, rsa_e_len, sig_len;
static uint32_t bootimg_addr, bootimg_len;

/*
ベリファイ内容
1. FormatVersion が 1 であること
2. AuthenticatedAttributes のPrintableStringが /bootであり、
INTEGERがboot.imgサイズと一致していること
3. SubjectPublicKeyでSignatureをデクリプトした結果が
boot.img + AuthenticatedAttributes
のSHA256ハッシュであること
4. グリーン条件として、SubjectPublicKeyがGoogleのものであること。
*/

int prepare_verify(uint32_t asn1_start, uint32_t bootimg_start, uint32_t bootimg_size,
				int recovery)
{
	uint8_t *p = (uint8_t *)asn1_start;
	uint8_t *q;
	int ret, i;
	uint8_t *cert;
	uint32_t length;
	uint8_t *pval;
	uint8_t *aa;	// AuthenticatedAttributes
	uint32_t aa_len;	// AuthenticatedAttributes length

	bootimg_addr = bootimg_start;
	if (!(p = asn1_obj_get(ASN1_TAG_SEQUENCE, p, &length)) ||
		// p == FormatVersion
		!(p = asn1_obj_get_skip(ASN1_TAG_INTEGER, p, &pval, &length)))
		// p == Certificate
		return 1;
	if (length != 1 || *pval != 1) {
		printf("Invalid format version\n");
		return 1;
	}
	cert = p;	// backup

	if (!(p = asn1_obj_get(ASN1_TAG_SEQUENCE, p, &length)) ||
		// p == Data
		!(p = asn1_obj_get(ASN1_TAG_SEQUENCE, p, &length)) ||
		// p == Version
		!(p = asn1_constructed_skip_all(p)) ||
		// p == SerialNumber
		!(p = asn1_obj_skip(ASN1_TAG_INTEGER, p)) ||
		// p == SignatureAlgorithm
		!(p = asn1_obj_skip(ASN1_TAG_SEQUENCE, p)) ||
		// p == Issuer
		!(p = asn1_obj_skip(ASN1_TAG_SEQUENCE, p)) ||
		// p == Validity
		!(p = asn1_obj_skip(ASN1_TAG_SEQUENCE, p)) ||
		// p == Subject
		!(p = asn1_obj_skip(ASN1_TAG_SEQUENCE, p)) ||
		// q == SubjectPublicKeyInfo
		!(p = asn1_obj_get(ASN1_TAG_SEQUENCE, p, &length)) ||
		// p == AlgorithmIdentifier
		!(p = asn1_obj_skip(ASN1_TAG_SEQUENCE, p)) ||
		// p == SubjectPublicKey
		!(p = asn1_obj_get(ASN1_TAG_BITSTRING, p, &length)))
		// p == encapsulated pubkey
		return 1;
	if (*p != 0) {
		printf("unused bit is not equal 0\n");
		return 1;
	}
	p++;
	if (!(p = asn1_obj_get(ASN1_TAG_SEQUENCE, p, &length)))
		// p == modules
		return 1;

	// compare to oemkey
	if (length != 0x10a)
		return 1;
	cm_secure_boot_set_pubkey(p - 2, 268);
	if (memcmp(p, oemkey, 0x10a) == 0)
		is_oemkey = 1;

	if (!(p = asn1_obj_get_skip(ASN1_TAG_INTEGER, p, &prsa_n, &rsa_n_len)) ||
		//p == exponent
		!(p = asn1_obj_get_skip(ASN1_TAG_INTEGER, p, &prsa_e, &rsa_e_len)))
		return 1;
#if 0
	printf("rsa_n_len = %d", rsa_n_len);
	for (i = 0; i < rsa_n_len; i++) {
		if ((i & 15) == 0)
			printf("\n");
		printf("%02x,", *(prsa_n + i));
	}
	printf("\n");
	printf("rsa_e_len = %d", rsa_e_len);
	for (i = 0; i < rsa_e_len; i++) {
		if ((i & 15) == 0)
			printf("\n");
		printf("%02x,", *(prsa_e + i));
	}
	printf("\n");
#endif
	if (!(p = asn1_obj_skip(ASN1_TAG_SEQUENCE, cert)))
		// p == AlgorithmIdentifier
		return 1;
	if (memcmp(p + 4, oid_sha256withrsa, sizeof(oid_sha256withrsa)) != 0) {
		printf("unsupported algorithm\n");
		return 1;
	}
	if (!(aa = asn1_obj_skip(ASN1_TAG_SEQUENCE, p)) ||
		// aa == AuthenticatedAttributes
		!(p = asn1_obj_get(ASN1_TAG_SEQUENCE, aa, &aa_len)))
		// p == PrintableString
		return 1;
	aa_len += (int)(p - aa);
	// copy AuthenticatedAttributes at the last of boot.img
	q = (uint8_t *)asn1_start;
//	printf("AuthenticatedAttributes =\n");
	for (i = 0; i < aa_len; i++) {
	//	printf("%02x ", *(aa + i));
		*(q + i) = *(aa + i);
	}
//	printf("\n");

	if (!(p = asn1_obj_get_skip(ASN1_TAG_PRINTABLE, p, &pval, &length)))
		return 1;
	if (recovery) {
		if (memcmp(pval, "/recovery", 9)) {
			printf("invalid partition\n");
			return 1;
		}
	} else {
		if (memcmp(pval, "/boot", 5)) {
			printf("invalid partition\n");
			return 1;
		}
	}
	if (!(p = asn1_obj_get_skip(ASN1_TAG_INTEGER, p, &pval, &length)))
		return 1;
	bootimg_len = 0;
	for (i = 0; i < length; i++) {
		bootimg_len <<= 8;
		bootimg_len += *pval;
		pval++;
	}
	if (bootimg_len != bootimg_size) {
		printf("bootimg size err %d %d\n", bootimg_len, bootimg_size);
		return 1;
	}
	bootimg_len += aa_len;
	if (!(p = asn1_obj_get_skip(ASN1_TAG_OCTETSTR, p, &psig, &sig_len)))
		return 1;
#if 0
	printf("Signature =");
	for (i = 0; i < sig_len; i++) {
		if ((i & 15) == 0)
			printf("\n");
		printf("%02x ", psig[i]);
	}
	printf("\n");
#endif
	return 0;
}

static uint32_t digest[8];

int verify_bootimg(void)
{
	int ret;
	RSA *rsa;

	get_hash(bootimg_addr, bootimg_len, ALG_SHA256, digest);

	rsa = RSA_new();

	if (rsa == NULL) {
		printf("RSA_new failed\n");
		return 0;
	}
	rsa->n = BN_new();
	if (rsa->n == NULL) {
		printf("BN_new failed1\n");
		return 0;
	}
	rsa->e = BN_new();
	if (rsa->e == NULL) {
		printf("BN_new failed2\n");
		return 0;
	}

	if (BN_bin2bn(prsa_n, 257, rsa->n) == NULL) {
		printf("BN_bin2bn failed1\n");
		return 0;
	}
	if (BN_bin2bn(prsa_e, 3, rsa->e) == NULL) {
		printf("BN_bin2bn failed2\n");
		return 0;
	}

	ret = RSA_verify(NID_sha256, (const uint8_t *)digest, 32, psig, 256, rsa);

	return ret;
}

