#!/bin/bash

BOARD=""
TOOL=""
ACPM="no"
ACPM_FRAMEWORK="framework.fw"
ACPM_PLUGIN="plugin1.fw"
ECT="no"
ECT_NAME="ect.app"
SOC=""
BL2_ADJUST_SIZE="0"
SG="yes"
SIGNATURE_PADDING="no"

if [ "$1" == "espresso7570_aarch64" ]
then
	BOARD="espresso7570"
	ACPM="yes"
	ECT="yes"
	ECT_NAME="ect_24kb.app"
	SOC="exynos7570"
	BL2_ADJUST_SIZE="65536"
	SIGNATURE_PADDING="yes"
	SG="no"
elif [ "$1" == "espresso7270_aarch64" ] || [ "$1" == "espresso7270_sip_aarch64" ]
then
	BOARD="espresso7570"
	ACPM="yes"
	ECT="yes"
	ECT_NAME="ect_wear_24kb.app"
	SOC="exynos7570"
	BL2_ADJUST_SIZE="65536"
	SIGNATURE_PADDING="yes"
	SG="no"
else
	echo % wrong board name
	echo % usage : ./build.sh [board name]
	echo % board list
	echo espresso7270_sip_aarch64
	echo espresso7270_aarch64
	echo espresso7570_aarch64
	exit -1
fi

rm -rf tools/*kb
rm -rf output
mkdir -p output

if [ $SG == "no" ]
then
make distclean $1_config
make -j8
mv spl/u-boot-spl output/u-boot-spl
mv spl/u-boot-spl.bin output/u-boot-spl.bin
mv spl/$BOARD-spl.bin output/$BOARD-spl.bin
else
make distclean $1_SG_config
make -j16
mv spl/u-boot-spl output/u-boot-spl.sg
mv spl/u-boot-spl.bin output/u-boot-spl.bin.sg
mv spl/$BOARD-spl.bin output/$BOARD-spl.bin.sg

make distclean $1_config
make -j16
mv spl/u-boot-spl output/u-boot-spl.org
mv spl/u-boot-spl.bin output/u-boot-spl.bin.org
mv spl/$BOARD-spl.bin output/$BOARD-spl.bin.org

cat output/$BOARD-spl.bin.org output/$BOARD-spl.bin.sg > output/$BOARD-spl.bin.tmp

if [ "$TOOL" == "" ]
then
	tools/adjust_size output/$BOARD-spl.bin.tmp output/$BOARD-spl.bin $BL2_ADJUST_SIZE
else
	tools/$TOOL output/$BOARD-spl.bin.tmp output/$BOARD-spl.bin
fi
fi

if [ $ACPM == "yes" ]
then
	cp output/$BOARD-spl.bin output/$BOARD-spl.bin.no_acpm
	cat output/$BOARD-spl.bin.no_acpm bin/acpm/$SOC/$ACPM_FRAMEWORK > output/$BOARD-spl.bin.acpm
	cat output/$BOARD-spl.bin.acpm bin/acpm/$SOC/$ACPM_PLUGIN > output/$BOARD-spl.bin
fi

if [ $ECT == "yes" ]
then
	cp output/$BOARD-spl.bin output/$BOARD-spl.bin.no_ect
	cat output/$BOARD-spl.bin.no_ect bin/ect/$SOC/$ECT_NAME > output/$BOARD-spl.bin
fi

if [ "$SIGNATURE_PADDING" == "yes" ]
then
	tools/adjust_size output/$BOARD-spl.bin output/$BOARD-spl.bin $BL2_ADJUST_SIZE
fi

mv u-boot.bin output

ls -l output
exit 0
