/*
 * Copyright (C) 2017 Samsung Electronics
 *
 * See file CREDITS for list of people who contributed to this
 * project.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */

#include <common.h>
#include <command.h>
#include <fastboot.h>

int do_scatter_load_boot(cmd_tbl_t * cmdtp, int flag, int argc, char *argv[])
{
	unsigned long long boot_addr, kernel_addr, dtb_addr, ramdisk_addr;
	struct fastboot_boot_img_hdr *fb_hdr;
	int kernel_offset;
	int dtb_offset;
	int ramdisk_offset;
	char initrd_size[32];

	if (argc != 5) goto usage;

	boot_addr = simple_strtoull(argv[1], NULL, 16);
	kernel_addr = simple_strtoull(argv[2], NULL, 16);
	ramdisk_addr = simple_strtoull(argv[3], NULL, 16);
	dtb_addr = simple_strtoull(argv[4], NULL, 16);

	fb_hdr = (struct fastboot_boot_img_hdr *)boot_addr;

	printf("page size: 0x%08x\n", fb_hdr->page_size);
	printf("kernel size: 0x%08x\n", fb_hdr->kernel_size);
	printf("ramdisk size: 0x%08x\n", fb_hdr->ramdisk_size);
	if (dtb_addr)
		printf("DTB size: 0x%08x\n", fb_hdr->second_size);

	kernel_offset = fb_hdr->page_size;
	ramdisk_offset = kernel_offset + ((fb_hdr->kernel_size + fb_hdr->page_size - 1) / fb_hdr->page_size) * fb_hdr->page_size;
	if (dtb_addr)
		dtb_offset = ramdisk_offset + ((fb_hdr->ramdisk_size + fb_hdr->page_size - 1) / fb_hdr->page_size) * fb_hdr->page_size;

	memcpy((void *)kernel_addr, (const void *)(boot_addr + kernel_offset), (size_t)fb_hdr->kernel_size);
	memcpy((void *)ramdisk_addr, (const void *)(boot_addr + ramdisk_offset), (size_t)fb_hdr->ramdisk_size);
	if (dtb_addr)
		memcpy((void *)dtb_addr, (const void *)(boot_addr + dtb_offset), (size_t)fb_hdr->second_size);

	sprintf(initrd_size, "0x%x", fb_hdr->ramdisk_size);
	setenv("rootfslen", initrd_size);
	setenv("recoverylen", initrd_size);

	return 0;

usage:
	printf("Usage:\n%s\n", cmdtp->usage);
	return -1;
}

U_BOOT_CMD(
	scatter_load_boot,	5,	0,	do_scatter_load_boot,
	"scatter_load_boot - scatter load kernel, dtb, ramdisk from boot/recovery.img",
	"scatter_load_boot {boot/recovery addr} {kernel addr} {dtb addr} {ramdisk addr}\n"
);
