/* drivers/video/exynos/7880/vpp/vpp.h
 *
 * header file for Samsung EXYNOS5 SoC series VPP driver.
 *
 * Copyright@ Samsung Electronics Co. LTD
 *
 * Manseok Kim <manseoks.kim@samsung.com> *
 * Lee HwangJae <hj-yo.lee@samsung.com> *
 *
 * This software is proprietary of Samsung Electronics.
 * No part of this software, either material or conceptual may be copied or distributed, transmitted,
 * transcribed, stored in a retrieval system or translated into any human or computer language in any form by any means,
 * electronic, mechanical, manual or otherwise, or disclosed
 * to third parties without the express written permission of Samsung Electronics.


 * Alternatively, this program is free software in case of open source projec;
 * you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.

*/


#ifndef VPP_H_
#define VPP_H_

#include <common.h>
#include <linux/err.h>

#include "regs-vpp.h"
#include "vpp_common.h"
#include "../../decon.h"

typedef enum {false, true} bool;
extern int decon_log_level;

#define vpp_err(fmt, ...)							\
	do {									\
		if (decon_log_level >= 3) 					\
			printf(fmt, ##__VA_ARGS__);				\
	} while (0)

#define vpp_info(fmt, ...)							\
	do {									\
		if (decon_log_level >= 5) 					\
			printf(fmt, ##__VA_ARGS__);				\
	} while (0)

#define vpp_dbg(fmt, ...)							\
	do {									\
		if (decon_log_level >= 6) 					\
			printf(fmt, ##__VA_ARGS__);				\
	} while (0)

struct vpp_params {
	void __iomem			*addr;
};

struct decon_frame {
	int x;
	int y;
	u32 w;
	u32 h;
	u32 f_w;
	u32 f_h;
};

struct decon_win_config {
	struct vpp_params		vpp_parm;
	enum vpp_pixel_format		format;
	/* source framebuffer coordinates */
	struct decon_frame		src;
	struct decon_frame		dst;
};

struct vpp_dev {
	int				id;
	void __iomem			*regs;
	struct decon_win_config		*config;
	u32				pbuf_num;
	u32				h_ratio;
	u32				v_ratio;
};

extern struct vpp_dev *vpp_for_decon;

static inline void vpp_to_scale_params(struct vpp_dev *vpp, struct vpp_size_param *p)
{
	struct decon_win_config *config = vpp->config;
	p->src_x = config->src.x;
	p->src_y = config->src.y;
	p->src_w = config->src.w;
	p->src_h = config->src.h;
	p->dst_w = config->dst.w;
	p->dst_h = config->dst.h;
	p->src_fw = config->src.f_w;
	p->src_fh = config->src.f_h;
	p->addr0 = config->vpp_parm.addr;
}

static inline struct vpp_dev *get_vpp_drvdata(u32 id)
{
	switch (id) {
	case 1:
		return vpp_for_decon;
	default:
		return NULL;
	}
}

static inline u32 vpp_read(u32 id, u32 reg_id)
{
	struct vpp_dev *vpp = get_vpp_drvdata(id);
	return readl(vpp->regs + reg_id);
}

static inline u32 vpp_read_mask(u32 id, u32 reg_id, u32 mask)
{
	u32 val = vpp_read(id, reg_id);
	val &= (~mask);
	return val;
}

static inline void vpp_write(u32 id, u32 reg_id, u32 val)
{
	struct vpp_dev *vpp = get_vpp_drvdata(id);
	writel(val, vpp->regs + reg_id);
}

static inline void vpp_write_mask(u32 id, u32 reg_id, u32 val, u32 mask)
{
	struct vpp_dev *vpp = get_vpp_drvdata(id);
	u32 old = vpp_read(id, reg_id);

	val = (val & mask) | (old & ~mask);
	writel(val, vpp->regs + reg_id);
}

/* CAL APIs list */
int vpp_set_config(u32 dev_id, unsigned long addr);
#endif
