/* drivers/video/exynos/decon_font.c
 *
 * SAMSUNG SoC FONT for EXYNOS8890.
 *
 * Copyright@ Samsung Electronics Co. LTD
 *
 * Manseok Kim <manseoks.kim@samsung.com> *
 * Seuni Park <seuni.park@samsung.com> *
 *
 * This software is proprietary of Samsung Electronics.
 * No part of this software, either material or conceptual may be copied or distributed, transmitted,
 * transcribed, stored in a retrieval system or translated into any human or computer language in any form by any means,
 * electronic, mechanical, manual or otherwise, or disclosed
 * to third parties without the express written permission of Samsung Electronics.


 * Alternatively, this program is free software in case of open source projec;
 * you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.

*/


#include "asm/types.h"
#include "decon_font.h"
#include <common.h>
#include <asm/arch/display.h>
#include "./panels/lcd_ctrl.h"

#ifdef CONFIG_DISPLAY_DRAWFONT
static int low = 50;
#define MAX_WIDTH (LCD_WIDTH / (FONT_X + 1))

int decon_fill_frame_buffer_ch(u32 *buf, u32 x, u8 *str,
		u32 color, u32 bg_color)
{
	int num = 0;
	int over_num = 0;

	if (!str)
		return -1;
	else
		num = strlen(str);

	do {
		num = decon_fill_all_char(buf, x, str, color, bg_color, num);
		if (num)
			str = str + MAX_WIDTH;
	} while(num > 0);

	return 0;
}

int decon_fill_all_char(u32 *buf, u32 x, u8 *str,
		u32 color, u32 bg_color, int num)
{
	int i = 0;
	int count = 0;
	struct decon_lcd *lcd_info = decon_get_lcd_info();

	if (num > MAX_WIDTH)
		count = MAX_WIDTH;
	else
		count = num;

	for (i = 0; i < count; i++) {
		if (decon_fill_one_char(buf, x + (i * FONT_X),
					lcd_info->xres, (*str++) - 32, low, color, bg_color)) {
			printf("This(%c) character is not supported\n", *(str-1));
		}
	}

	low += FONT_Y + 13;
	num = num - MAX_WIDTH;

	return num;
}
#endif


static u32 calc_mixcolor(u32 fcolor, u32 bcolor, u32 ratio)
{
	int i;
	u32 fval, bval, val = 0;

	for (i = 0; i < 3; i++) {
		val <<= 8;
		fval = fcolor & 0xff;
		bval = bcolor & 0xff;
		val |= (fval * ratio + bval * (256 - ratio)) / 256;
		fcolor >>= 8;
		bcolor >>= 8;
	}
	val |= 0xff000000;
	return val;
}

/* 2bit font */
static void decon_fill_one_char2(u32 *buf, u32 x, u32 pitch, int num,
		int low, u32 color, u32 bg_color, const u32 *pfont)
{
	int i, j;
	u32 offset = FONT_Y * num;
	u32 *ptr;
	u32 val;
	u32 ctable[4];

	ctable[0] = bg_color;
	ctable[1] = calc_mixcolor(color, bg_color, 84);
	ctable[2] = calc_mixcolor(color, bg_color, 168);
	ctable[3] = color;

	for (i = 0; i < FONT_Y; i++) {
		ptr = buf + ((i + low) * pitch) + x;
		val = pfont[offset];
		for (j = 0; j < FONT_X; j++) {
			ptr[11 - j] = ctable[val & 3];
			val >>= 2;
		}
		offset++;
	}
}


static int decon_fill_one_char(u32 *buf, u32 x, u32 pitch, int num,
		int low, u32 color, u32 bg_color)
{
	if (num < 0 || num > 90)
		return -1;
	if (num >= 65)
		decon_fill_one_char2(buf, x, pitch, num - 65, low, color, bg_color, font_lower);
	else if (num >= 33)
		decon_fill_one_char2(buf, x, pitch, num - 33, low, color, bg_color, font_upper);
	else
		decon_fill_one_char2(buf, x, pitch, num, low, color, bg_color, font_etc);

	return 0;
}

extern u32 *win_fb0;

void decon_text(u32 color, u32 bg_color, u32 x, u32 y, const char *fmt, ...)
{
	va_list args;
	struct decon_lcd *lcd_info = decon_get_lcd_info();
	char printbuffer[24];
	uint num;
	int i;

	va_start(args, fmt);
	num = vscnprintf(printbuffer, sizeof(printbuffer), fmt, args);
	va_end(args);

	for (i = 0; i < num; i++) {
		decon_fill_one_char(win_fb0, x, lcd_info->xres,
				printbuffer[i] - 32, y, color, bg_color);
		x += FONT_X;
	}
}
