/* drivers/video/exynos/panels/s6e8aa0_lcd_ctrl.c
 *
 * Samsung SoC MIPI LCD CONTROL functions
 *
 * Copyright@ Samsung Electronics Co. LTD
 *
 * Manseok Kim, <manseoks.kim@samsung.com> *
 *
 * This software is proprietary of Samsung Electronics.
 * No part of this software, either material or conceptual may be copied or distributed, transmitted,
 * transcribed, stored in a retrieval system or translated into any human or computer language in any form by any means,
 * electronic, mechanical, manual or otherwise, or disclosed
 * to third parties without the express written permission of Samsung Electronics.


 * Alternatively, this program is free software in case of open source projec;
 * you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.

*/

#include <asm/arch/mipi_dsim.h>
#include "s6e8aa0_param.h"
#include "lcd_ctrl.h"
#include "../dsim.h"

#define msleep(a) udelay(a * 1000)

#define VIDEO_MODE      1
#define COMMAND_MODE    0


struct decon_lcd s6e8aa0_lcd_info = {
	/* Only availaable VIDEO MODE */
	.mode = DECON_VIDEO_MODE,
#ifdef CONFIG_CPU_EXYNOS7570
	.decon_hbp = 0x1,
	.decon_hfp = 0x223,
	.decon_hsa = 0x1,

	.decon_vbp = 0x1,
	.decon_vfp = 0xD,
	.decon_vsa = 0x2,

	.dsim_hbp = 0x60,
	.dsim_hfp = 0x4C,
	.dsim_hsa = 0x01,

	.dsim_vbp = 0x1,
	.dsim_vfp = 0xD,
	.dsim_vsa = 0x2,
#else
	.hbp = 0x1,
	.hfp = 0x1,
	.hsa = 0x02,

	.vbp = 0x1,
	.vfp = 0xD,
	.vsa = 0x2,

#endif
	.xres = 720,
	.yres = 1280,

	.width = 71,
	.height = 114,

	/* Mhz */
	.hs_clk = 600,
	.esc_clk = 20,

	.dphy_pms.p = 2,
	.dphy_pms.m = 92,
	.dphy_pms.s = 1,

	.fps = 60,
	.mic_enabled = 0,
	.mic_ratio = MIC_VER_2_0,
	.mic_ver = MIC_VER_1_2,
};

struct decon_lcd *decon_get_lcd_info(void)
{
	return &s6e8aa0_lcd_info;
}

/*
 * 8AA0 lcd init sequence
 */

void lcd_init(u32 id, struct decon_lcd * lcd)
{
	/*level1_command (to be added)*/
	while(dsim_wr_data(id, MIPI_DSI_DCS_LONG_WRITE,
		(unsigned long) apply_level_1_key,
		ARRAY_SIZE(apply_level_1_key)) == -1)
		dsim_err("failed to send apply_level_1_key_command.\n");

	/*level2_command*/
	while(dsim_wr_data(id, MIPI_DSI_DCS_LONG_WRITE,
		(unsigned long) apply_level_2_key,
		ARRAY_SIZE(apply_level_2_key)) == -1)
		dsim_err("failed to send apply_level_2_key_command.\n");

	/*sleep out */
	while(dsim_wr_data(id, MIPI_DSI_DCS_LONG_WRITE,
		(unsigned long) sleep_out,
		ARRAY_SIZE(sleep_out)) == -1)
		dsim_err("failed to send sleep_out_command.\n");

	/*5ms delay */
	mdelay(100);

	/*panel_condition_set */
	while(dsim_wr_data(id, MIPI_DSI_DCS_LONG_WRITE,
		(unsigned long) panel_condition_set,
		ARRAY_SIZE(panel_condition_set)) == -1)
		dsim_err("failed to send panel_condition_set_command.\n");

	/*panel_condition_update */
	while(dsim_wr_data(id, MIPI_DSI_DCS_LONG_WRITE,
		(unsigned long) panel_condition_update,
		ARRAY_SIZE(panel_condition_update)) == -1)
		dsim_err("failed to send panel_condition_update_command.\n");

	/*gamma_condition_set */
	while(dsim_wr_data(id, MIPI_DSI_DCS_LONG_WRITE,
		(unsigned long) gamma_condition_set,
		ARRAY_SIZE(gamma_condition_set)) == -1)
		dsim_err("failed to send gamma_condition_set_command.\n");

	/*gamma_condition_update */
	while(dsim_wr_data(id, MIPI_DSI_DCS_LONG_WRITE,
		(unsigned long) gamma_update,
		ARRAY_SIZE(gamma_update)) == -1)
		dsim_err("failed to send gamma_update_command.\n");

	/*source control */
	while(dsim_wr_data(id, MIPI_DSI_DCS_LONG_WRITE,
		(unsigned long) etc_set_source_ctrl,
		ARRAY_SIZE(etc_set_source_ctrl)) == -1)
		dsim_err("failed to etc_set_source_ctrl_command.\n");

	/*pentile control */
	while(dsim_wr_data(id, MIPI_DSI_DCS_LONG_WRITE,
		(unsigned long) etc_set_pentile_ctrl,
		ARRAY_SIZE(etc_set_pentile_ctrl)) == -1)
		dsim_err("failed to send etc_set_pentile_ctrl_command.\n");

	/*nvm setting */
	while(dsim_wr_data(id, MIPI_DSI_DCS_LONG_WRITE,
		(unsigned long) elvss_NVM_set,
		ARRAY_SIZE(elvss_NVM_set)) == -1)
		dsim_err("failed to send elvss_NVM_set_command.\n");

	/*power control */
	while(dsim_wr_data(id, MIPI_DSI_DCS_LONG_WRITE,
		(unsigned long) etc_set_power_ctrl,
		ARRAY_SIZE(etc_set_power_ctrl)) == -1)
		dsim_err("failed to send etc_set_power_ctrl_command.\n");

	/*dynamic elvss control */
	while(dsim_wr_data(id, MIPI_DSI_DCS_LONG_WRITE,
		(unsigned long) elvss_ctrl_set,
		ARRAY_SIZE(elvss_ctrl_set)) == -1)
		dsim_err("failed to send dyanmic_elvss_ctrl_set_command.\n");

	/*acl control1 */
	while(dsim_wr_data(id, MIPI_DSI_DCS_LONG_WRITE,
		(unsigned long) acl_ctrl1,
		ARRAY_SIZE(acl_ctrl1)) == -1)
		dsim_err("failed to send acl_ctrl1_command.\n");

	/*acl control2 */
	while(dsim_wr_data(id, MIPI_DSI_DCS_LONG_WRITE,
		(unsigned long) acl_ctrl2,
		ARRAY_SIZE(acl_ctrl2)) == -1)
		dsim_err("failed to send acl_ctrl2_command.\n");

	/* 120ms delay */
	mdelay(200);

	/* display on */
	dsim_wr_data(id, MIPI_DSI_DCS_SHORT_WRITE,
		0x29, 0);
}


void lcd_enable(void)
{
}

void lcd_disable(void)
{
}
