/*
 *	drivers/video/exynos/panels/sharp_lcd_ctrl.c
 */

#include <asm/arch/mipi_dsim.h>
#include "sharp_lcd_param.h"
#include "lcd_ctrl.h"
#include "../dsim.h"

#define msleep(a) udelay(a * 1000)

#define VIDEO_MODE      1
#define COMMAND_MODE    0

#define ID		0

struct decon_lcd sharp_lcd_info = {

	.mode = DECON_MIPI_COMMAND_MODE,

	.decon_hbp = 7,
	.decon_hfp = 1,
	.decon_hsa = 1,

	.decon_vbp = 10,
	.decon_vfp = 2,
	.decon_vsa = 1,

	.dsim_hbp = 1,
	.dsim_hfp = 1,
	.dsim_hsa = 1,

	.dsim_vbp = 10,
	.dsim_vfp = 2,
	.dsim_vsa = 1,

	.xres = 320,
	.yres = 300,

	.width = 34,
	.height = 32,
	.dispif_w = 300,
	.dispif_h = 320,

	/* Mhz */
	.hs_clk = 172,
	.esc_clk = 20,

	.dphy_pms.p = 2,
	.dphy_pms.m = 106,
	.dphy_pms.s = 3,

	.fps = 60,
};

struct decon_lcd *decon_get_lcd_info(void)
{
	return &sharp_lcd_info;
}

/*
 * sharp lcd init sequence
 */
static void sharp_mipi_select_page(uint8_t page)
{
	int ret;

	ret = dsim_wr_data(ID, MIPI_DSI_DCS_SHORT_WRITE,
			0xFF, page);
	if (ret)
		dsim_err("failed to select page 0x%02x: %d\n", page, ret);
	udelay(300);
}
#if 0
static int sharp_get_display_id1(void)
{
	uint8_t read_data;
	int ret;

	ret = dsim_rd_data(ID, MIPI_DSI_RX_DCS_SHORT_READ_RESPONSE_1BYTE, 0xA1, 0x01, &read_data);
	if (ret) {
		dsim_err("failed to read display id1: %d\n", ret);
		return -1;
	}

	return read_data;
}

static int sharp_get_display_id2(void)
{
	uint8_t read_data;
	int ret;

	ret = dsim_rd_data(ID, MIPI_DSI_RX_DCS_SHORT_READ_RESPONSE_1BYTE, 0xC5, 0x01, &read_data);
	if (ret) {
		dsim_err("failed to read display id2: %d\n", ret);
		return -1;
	}

	return read_data;
}

static void sharp_init_commands(int id1)
{
	int i, ret, id2;

	sharp_mipi_select_page(0x24);
	id2 = sharp_get_display_id2();
	for (i = 0; i < ARRAY_SIZE(SHARP_INIT_CMDS1); i++) {
		ret = dsim_wr_data(ID, MIPI_DSI_DCS_SHORT_WRITE,
				SHARP_INIT_CMDS1[i][0], SHARP_INIT_CMDS1[i][1]);
		if (ret)
			dsim_err("failed to write CMDS1[%d]: %d\n", i, ret);
	}

	sharp_mipi_select_page(0x20);
	if (id1 == 0x01 || id2 == 0xA1) {
		ret = dsim_wr_data(ID, MIPI_DSI_DCS_SHORT_WRITE,
				0xFB, 0x01);
		if (ret)
			dsim_err("failed to write extra CMDS2: %d\n", ret);
	}
	for (i = 0; i < ARRAY_SIZE(SHARP_INIT_CMDS2); i++) {
		ret = dsim_wr_data(ID, MIPI_DSI_DCS_SHORT_WRITE,
				SHARP_INIT_CMDS2[i][0], SHARP_INIT_CMDS2[i][1]);
		if (ret)
			dsim_err("failed to write CMDS2[%d]: %d\n", i, ret);
	}

	sharp_mipi_select_page(0x21);
	if (id1 == 0x01 || id2 == 0xA1) {
		ret = dsim_wr_data(ID, MIPI_DSI_DCS_SHORT_WRITE,
				0xFB, 0x01);
		if (ret)
			dsim_err("failed to write extra CMDS3: %d\n", ret);
	}
	for (i = 0; i < ARRAY_SIZE(SHARP_INIT_CMDS3); i++) {
		ret = dsim_wr_data(ID, MIPI_DSI_DCS_SHORT_WRITE,
				SHARP_INIT_CMDS3[i][0], SHARP_INIT_CMDS3[i][1]);
		if (ret)
			dsim_err("failed to write CMDS3[%d]: %d\n", i, ret);
	}

	sharp_mipi_select_page(0xE0);
	ret = dsim_wr_data(ID, MIPI_DSI_DCS_SHORT_WRITE,
			0x42, 0x21);
	if (ret)
		dsim_err("failed to write CMDS4: %d\n", ret);
}
#endif
static void sharp_power_on_sequence(void)
{
	int ret;

	unsigned char PWRON_CMD_01[2]	= {0xFB, 0x01 };
	unsigned char PWRON_CMD_02[2]	= {0xB3, 0x00 };
	unsigned char PWRON_CMD_03[2]	= {0xBB, 0x10 };
	unsigned char PWRON_CMD_04[2]	= {0x3A, 0x06 }; /* PIXEL FORMAT */
	unsigned char PWRON_CMD_05[6]	= {0x3B, 0x00, 0x08, 0x08, 0x20, 0x20 };
	unsigned char PWRON_CMD_06[2]	= {0x35, 0x00 }; /* TEAR ON */
	unsigned char PWRON_CMD_07[2]	= {0x11, 0x00 }; /* EXIT SLEEP MODE */
	unsigned char PWRON_CMD_08[2]	= {0x29, 0x00 }; /* DISPLAY ON */


	ret = dsim_wr_data(ID, MIPI_DSI_DCS_SHORT_WRITE,
			PWRON_CMD_01[0], PWRON_CMD_01[1]);
	if (ret)
		dsim_err("failed to write CMD01: %d\n", ret);

	ret = dsim_wr_data(ID, MIPI_DSI_DCS_SHORT_WRITE,
			PWRON_CMD_02[0], PWRON_CMD_02[1]);
	if (ret)
		dsim_err("failed to write CMD02: %d\n", ret);

	ret = dsim_wr_data(ID, MIPI_DSI_DCS_SHORT_WRITE,
			PWRON_CMD_03[0], PWRON_CMD_03[1]);
	if (ret)
		dsim_err("failed to write CMD03: %d\n", ret);

	ret = dsim_wr_data(ID, MIPI_DSI_DCS_SHORT_WRITE,
			PWRON_CMD_04[0], PWRON_CMD_04[1]);
	if (ret)
		dsim_err("failed to write CMD04: %d\n", ret);

	ret = dsim_wr_data(ID, MIPI_DSI_DCS_LONG_WRITE,
			(unsigned long)PWRON_CMD_05, ARRAY_SIZE(PWRON_CMD_05));
	if (ret)
		dsim_err("failed to write CMD05: %d\n", ret);

	ret = dsim_wr_data(ID, MIPI_DSI_DCS_SHORT_WRITE,
			PWRON_CMD_06[0], PWRON_CMD_06[1]);
	if (ret)
		dsim_err("failed to write CMD06: %d\n", ret);

	ret = dsim_wr_data(ID, MIPI_DSI_DCS_SHORT_WRITE,
			PWRON_CMD_07[0], PWRON_CMD_07[1]);
	if (ret)
		dsim_err("failed to write CMD07: %d\n", ret);
	msleep(130);

	ret = dsim_wr_data(ID, MIPI_DSI_DCS_SHORT_WRITE,
			PWRON_CMD_08[0], PWRON_CMD_08[1]);
	if (ret)
		dsim_err("failed to write CMD08: %d\n", ret);
	msleep(50);
}

void lcd_init(u32 id, struct decon_lcd *lcd)
{
	sharp_mipi_select_page(0x10);
	//ret = sharp_get_display_id1();
	/* if device is older than PVT */
	//if (ret != 0x02) {
	//	sharp_init_commands(ret);
	//	sharp_mipi_select_page(0x10);
	//}

	sharp_power_on_sequence();
}

void lcd_enable(void)
{
}

void lcd_disable(void)
{
}
